package net.comp_lot.lib.imagelib;

import java.awt.Frame;
import java.awt.Image;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

import javax.swing.JFrame;
import javax.swing.WindowConstants;

/**
 * 表示するためのクラス
 * @author pochi
 */
public class ImageView {
	private JFrame frame;
	private ImagePanel panel;
	private Map<MouseListener, MouseListener.ListenerAdapter> mlMap = new HashMap<>();	
	private boolean forcePaint = false;
	
	/**
	 * デフォルトのコンストラクタ
	 * @param title ウィンドウのタイトル
	 */
	public ImageView(String title) {
		frame = new JFrame();
		frame.setSize(800, 600);//大きさ
		frame.setExtendedState(Frame.MAXIMIZED_BOTH);//最大化
		frame.setTitle(title);//タイトル
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);//×押したときの動作
		panel = new ImagePanel();//表示するやつ本体
		frame.add(panel);//ウィンドウに本体ぶち込む
	}
	
	/**
	 * ウインドウを表示
	 */
	public void show() {
		frame.setVisible(true);
	}
	
	/**
	 * ウィンドウを閉じる
	 */
	public void close() {
		frame.dispose();
	}
	
	/**
	 * 描画の更新
	 */
	public void draw() {
		if (forcePaint)
			panel.forcePaint();
		else
			panel.repaint();
	}
	
//	/**
//	 * 描画情報用のリストを取得
//	 * @return 描画情報用のリスト
//	 */
//	private List<CustomImageData> getDataList() {
//		return panel.getImageDataList();
//	}
	
	/**
	 * 描画データの更新を行う。このメソッドの外で行うべきではない。
	 * @param function 更新する関数
	 */
	public void dataUpdate(Consumer<List<DrawableData>> function) {
		panel.startDataUpdate();
		function.accept(panel.getImageDataList());
		panel.endDataUpdate();
	}
	
	/**
	 * アスペクト比を設定する。
	 * 以後、表示領域はその内部として扱われる
	 * @param width 横
	 * @param height 縦
	 */
	public void setAspectRatio(int width, int height) {
		panel.setAspectRatio(width, height);
	}
	
	/**
	 * アスペクト比をウィンドウに合わせた値に戻す
	 */
	public void resetAspectRatio() {
		panel.resetAspectRatio();
	}
	
	/**
	 * draw実行時に確実に描画が行われるようにする。
	 * スピード感のある場面で推奨
	 * @param force 強制描画
	 */
	public void setForcePaint(boolean force) {
		forcePaint = force;
	}
	
	/**
	 * @return 表示領域の情報を取得
	 */
	public PanelInfo getPanelInfo() {
		return panel.info;
	}
	
	/**
	 * ウィンドウのタイトルを変更する
	 * @param title ウィンドウのタイトル
	 */
	public void setTitle(String title) {
		frame.setTitle(title);
	}
	
	/**
	 * ウィンドウのアイコンを設定する
	 * @param image アイコン画像
	 */
	public void setIcon(Image image) {
		frame.setIconImage(image);
	}
	
	/**
	 * キーリスナーを登録
	 * @param listener キーリスナー
	 */
	public void addKeyListener(KeyListener listener) {
		frame.addKeyListener(listener);
	}
	
	/**
	 * キーリスナーを削除
	 * @param listener キーリスナー
	 */
	public void removeKeyListener(KeyListener listener) {
		frame.removeKeyListener(listener);
	}
	
	/**
	 * マウスリスナーを登録
	 * @param listener マウスリスナー
	 */
	public void addMouseListener(MouseListener listener) {
		MouseListener.ListenerAdapter a = new MouseListener.ListenerAdapter(listener, this);
		mlMap.put(listener, a);
		panel.addMouseListener(a);
		panel.addMouseMotionListener(a);
		panel.addMouseWheelListener(a);
	}
	
	/**
	 * マウスリスナーを削除
	 * @param listener マウスリスナー
	 */
	public void removeMouseListener(MouseListener listener) {
		MouseListener.ListenerAdapter a = mlMap.remove(listener);
		panel.removeMouseListener(a);
		panel.removeMouseMotionListener(a);
		panel.removeMouseWheelListener(a);
	}
	
	/**
	 * ×が押された時の処理
	 * @param listener 実行する処理
	 */
	public void setOnClosing(Runnable listener) {
		WindowListener l = new WindowListener() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				listener.run();
			}
			
			@Override
			public void windowActivated(WindowEvent arg0) {}
			@Override
			public void windowClosed(WindowEvent arg0) {}
			@Override
			public void windowDeactivated(WindowEvent arg0) {}
			@Override
			public void windowDeiconified(WindowEvent arg0) {}
			@Override
			public void windowIconified(WindowEvent arg0) {}
			@Override
			public void windowOpened(WindowEvent arg0) {}
		};
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.addWindowListener(l);
	}
}
