package net.comp_lot.lib.imagelib;

import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

/**
 * マウスのイベントを処理するクラス用のインターフェイス
 * @author pochi
 */
public interface MouseListener {
	/**
	 * 押して離したとき
	 */
	public static final int CLICK = 1;
	/**
	 * 押したとき
	 */
	public static final int PRESS = 2;
	/**
	 * 離したとき
	 */
	public static final int RELEASE = 3;
	/**
	 * 動かしたとき、ボタンが押してある場合も
	 */
	public static final int MOVE = 4;
	
	/**
	 * 仲介用
	 * @author pochi
	 */
	static class ListenerAdapter extends MouseAdapter {
		private MouseListener listener;
		private ImageView view;
		
		// クリック判定用
		private int[] px = new int[6], py = new int[6];
		
		ListenerAdapter(MouseListener l, ImageView view) {
			listener = l;
			this.view = view;
		}
		
		@Override
		public void mousePressed(MouseEvent e) {
			if (e.getButton() > 5)
				return;
			px[e.getButton()] = e.getX();
			py[e.getButton()] = e.getY();
			onMouseEvent(PRESS, e.getX(), e.getY(), e.getButton());
			e.consume();
		}
		@Override
		public void mouseReleased(MouseEvent e) {
			if (e.getButton() > 5)
				return;
			if (getDist(e.getX(), e.getY(), e.getButton()) < 50)
				onMouseEvent(CLICK, e.getX(), e.getY(), e.getButton());
			onMouseEvent(RELEASE, e.getX(), e.getY(), e.getButton());
			e.consume();
		}
		@Override
		public void mouseMoved(MouseEvent e) {
			int num = 0, button = 0;
			boolean[] buttons = new boolean[6];
			for (int i=1;i<6;i++) {
				int mask = InputEvent.getMaskForButton(i);
				buttons[i] = (e.getModifiersEx() & mask) == mask;
				if (buttons[i]) {
					button = i;
					num++;
				}
			}
			if (num == 0)
				onMouseEvent(MOVE, e.getX(), e.getY(), 0);
			if (num == 1) {
				if (getDist(e.getX(), e.getY(), button) >= 50) {
					px[button] = -16000;
					py[button] = -16000;
					onMouseEvent(MOVE, e.getX(), e.getY(), button);
				}
			} else
				return;
			e.consume();
		}
		@Override
		public void mouseDragged(MouseEvent e) {
			mouseMoved(e);
		}
		@Override
		public void mouseWheelMoved(MouseWheelEvent e) {
			listener.onMouseWheelEvent(e.getWheelRotation());
		}
		
		private void onMouseEvent(int type, int x, int y, int button) {
			PanelInfo info = view.getPanelInfo();
			listener.onMouseEvent(type, x - info.getCorrectX(), y - info.getCorrectY(), button);
		}
		
		private int getDist(int x, int y, int button) {
			return sqrt(px[button] - x) + sqrt(py[button] - y);
		}
		private static int sqrt(int i) {
			return i * i;
		}
	}
	
	/**
	 * マウスのイベントを受け取ります
	 * @param type 動作のタイプ
	 * @param x マウスの位置
	 * @param y マウスの位置
	 * @param button 操作のあったボタン、0(NO_BUTTON)と1～5
	 */
	public void onMouseEvent(int type, int x, int y, int button);
	
	/**
	 * ホイールのイベントを受け取ります
	 * @param rotate 回転の大きさ、下(手前)のときに正
	 */
	public void onMouseWheelEvent(int rotate);
}
