package net.comp_lot.lib.imagelib;

import java.awt.Graphics;

import javax.swing.JPanel;

/**
 * 絶対指定のデータを割合で指定できるようにするためのアダプター
 * @author pochi
 * @param <T> データの型
 */
public class RatioDataAdapter<T extends AbsoluteDrawableData> extends DrawableData{
	private T data;
	private float x, y, width, height;
	private int panelWidth = 0, panelHeight = 0;
	private boolean setPlaceCenter = false;
	
	/**
	 * @param data 中身のデータ
	 */
	public RatioDataAdapter(T data) {
		this.data = data;
	}
	
	/**
	 * @return 中身のデータ
	 */
	public T get() {
		return data;
	}
	
	/**
	 * それぞれ0.0f~1.0fで設定
	 * @param width 幅
	 * @param height 高さ
	 */
	public void setSize(float width, float height) {
		this.width = width;
		this.height = height;
		data.setSizeAbsolute((int) (panelWidth*width), (int) (panelHeight*height));
	}
	
	/**
	 * それぞれ0.0f~1.0fで設定
	 * @param x 左上のx座標
	 * @param y 左上のy座標
	 */
	public void setPlace(float x, float y) {
		this.x = x;
		this.y = y;
		setPlaceCenter = false;
		data.setPlaceAbsolute((int) (panelWidth*x), (int) (panelHeight*y));
	}
	
	/**
	 * それぞれ0.0f~1.0fで設定
	 * @param x 中央のx座標
	 * @param y 中央のy座標
	 */
	public void setPlaceCenter(float x, float y) {
		this.x = x;
		this.y = y;
		setPlaceCenter = true;
		data.setPlaceAbsolute(getAbsoluteX(x-width*0.5f), getAbsoluteY(y-height*0.5f));
	}
	
	/**
	 * 画面の大きさが変更されたときに呼ばれる
	 */
	protected void updateSize() {}

	@Override
	protected void draw(Graphics g, PanelInfo info, JPanel panel) {
		if (info.getWidth() != panelWidth || info.getHeight() != panelHeight) {
			panelWidth = info.getWidth();
			panelHeight = info.getHeight();
			data.setSizeAbsolute(getAbsoluteX(width), getAbsoluteY(height));
			if (setPlaceCenter)
				data.setPlaceAbsolute(getAbsoluteX(x-width*0.5f), getAbsoluteY(y-height*0.5f));
			else
				data.setPlaceAbsolute(getAbsoluteX(x), getAbsoluteY(y));
			updateSize();
		}
		data.callDraw(g, info, panel);
	}
	
	/**
	 * 割合を実際の大きさに変換する
	 * @param x 割合
	 * @return 実際の大きさ
	 */
	protected int getAbsoluteX(float x) {
		return (int) (panelWidth * x);
	}
	/**
	 * 割合を実際の大きさに変換する
	 * @param y 割合
	 * @return 実際の大きさ
	 */
	protected int getAbsoluteY(float y) {
		return (int) (panelHeight * y);
	}

}
