package net.comp_lot.lib.imagelib;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;

/**
 * 文字を表示するためのクラス
 * @author pochi
 */
public class TextData extends AbsoluteDrawableData{
	/**
	 * 左寄せ
	 */
	public static final int ALIGN_LEFT = 0;
	/**
	 * 中央寄せ
	 */
	public static final int ALIGN_CENTER = 1;
	/**
	 * 右寄せ
	 */
	public static final int ALIGN_RIGHT = 2;
	
	private int size;
	private String[] text;
	private int align = ALIGN_LEFT;
	private ImageData background;
	private Color color;
	
	/**
	 * 文字列を指定する。
	 * 自動で改行が入ることがある。
	 * @param text 行ごとの文字列。
	 */
	public void setText(String...text) {
		List<String> list = new ArrayList<>();
		for (String s : text) {
			int a = 0, b = s.indexOf("\n", a);
			while (b >= 0) {
				list.add(s.substring(a,b));
				a = b + 1;
				b = s.indexOf("\n", a);
			}
			if (a != s.length())
				list.add(s.substring(a));
		}
		this.text = list.toArray(new String[list.size()]);
	}
	
	/**
	 * 文字の大きさを指定する。
	 * @param size 大きさ
	 */
	public void setTextSize(int size) {
		this.size = size;
	}
	
	/**
	 * 文字をどう寄せるか指定する
	 * @param align 寄せ方
	 */
	public void setAlign(int align) {
		this.align = align;
	}

	@Override
	protected void draw(Graphics g, PanelInfo info, JPanel panel) {
		if (background != null) {
			background.setPlaceAbsolute(x, y);
			background.setSizeAbsolute(width, height);
			background.callDraw(g, info, panel);
		}
		int startX = x + info.getCorrectX();
		int startY = y + info.getCorrectY();
		g.setFont(new Font(Font.SERIF, Font.PLAIN, size));
		int line = 0;
		for (String s : text) {
			line += drawString(g, s, startX, startY, width, line);
		}
	}
	
	/**
	 * 背景を設定。nullなら消去
	 * @param back 背景
	 */
	public void setBackground(BufferedImage back) {
		if (back != null) {
			background = new ImageData(back);
		} else {
			background = null;
		}
	}
	
	/**
	 * 文字の色を設定する。
	 * nullなら黒
	 * @param color 色
	 */
	public void setColor(Color color) {
		this.color = color;
	}
	
	/**
	 * 文字を描画。
	 * 必要なら改行。
	 */
	private int drawString(Graphics g, String text, int startX, int startY, int width, int line) {
		FontMetrics fontmetrics = g.getFontMetrics();
		if (fontmetrics.getHeight()*(line+1) > height)
			return 0;
		int textWidth = fontmetrics.stringWidth(text);
		if (textWidth > width) {
			int useLine = 0;
			int last = text.length() - 1;
			while (fontmetrics.stringWidth(text.substring(0, last)) > width) {
				last--;
			}
			if (last <= 0)
				return 0;
			useLine += drawString(g, text.substring(0, last), startX, startY, width, line);
			useLine += drawString(g, text.substring(last), startX, startY, width, line+useLine);
			return useLine;
		}
		if (color != null)
			g.setColor(color);
		switch (align) {
		case ALIGN_LEFT:
			g.drawString(text, startX, startY + fontmetrics.getAscent() + fontmetrics.getHeight()*line);
			break;
		case ALIGN_CENTER:
			g.drawString(text, startX+(width-textWidth)/2, startY + fontmetrics.getAscent() + fontmetrics.getHeight()*line);
			break;
		case ALIGN_RIGHT:
			g.drawString(text, startX+(width-textWidth), startY + fontmetrics.getAscent() + fontmetrics.getHeight()*line);
			break;
		}
		if (color != null)
			g.setColor(Color.BLACK);
		return 1;
	}

}
